/*
 * $Id: about.C,v 1.10 1996/12/01 22:50:25 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1994, 1995, 1996  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <htmlfile.H>

#include "version.H"
#include "addvutil.H"
#include "setaddv.H"
#include "nextroom.H"
#include "stat.H"
#include "backlink.H"

/***************************************************************************/

char *urlpath;   // The path relative to the server root
char *filepath;  // The absolute path in the file system
char *page;

HtmlFile *f;

/***************************************************************************/

void copyright_info(){
  f->header( 2, "Copyright Information" );
  f->puts( "This is <b>Addventure</b> version " );
  f->puts( VERSION_NUM );
  f->puts( " - " );
  f->puts( VERSION_DATE );
  f->para();

  f->puts( "Game Copyright (c) 1994-1996 by Allen S. Firstenberg.\n" );
  f->puts( "Individual segments copyrighted by their authors. ");
  f->puts( "Implicit permission to use given to the game administrators.\n" );
}

//---------------------------------------------------------------------------

void running_info_line(){

  // Setting info
  SetAddv setting( filepath );
  int   running        = setting.running();

  if( !running ){
    char *script = getenv( "SCRIPT_NAME" );
    char *urlpath = getenv( "PATH_INFO" );
    char *url = make_fname( script, urlpath );
    char *fullurl = make_url( NULL, NULL, url, "runinfo" );
    f->puts( "<B>The game is not running.</B> " );
    f->link( fullurl, "Further information may be available." );
    f->para();
  }
  
}

void running_info(){

  // Setting info
  SetAddv setting( filepath );
  int   running        = setting.running();
  char *headfile       = setting.getPathLabel( "file-error-closed-head" );
  char *tailfile       = setting.getPathLabel( "file-error-closed-tail" );

  f->header( 2, "Game Operations" );

  if( running ){
    f->puts( "The game is running!\n" );
  } else {

    f->puts( "<B>The game is currently shut down.</B>\n" );
    
    // Print a header
    if( headfile ){
      f->para();
      f->rawfile( headfile );
      f->para();
    }
  
    // Print a trailer
    if( tailfile ){
      f->para();
      f->rawfile( tailfile );
      f->para();
    }
  
  }
}

//---------------------------------------------------------------------------

void game_info(){
  
  SetAddv setting( filepath );
  char *title          = setting.game_title();
  char *version        = setting.version();
  char *started        = setting.date_started();
  char *administrator  = setting.contact();
  char *roomw          = setting.roomword();
  char *response       = setting.mailhelper();
  char *list           = setting.listhelper();
  int   mailsig        = setting.mailsig();
  int   backlink       = setting.backlink();
  int   listlinks      = setting.listlinks();
  int   maxopt         = setting.maxoptions();

  // A title for this secion
  if( title ){
    f->header( 2, title );
  } else {
    f->header( 2, "About this game" );
  }

  running_info_line();

  // When it was created with what version
  f->puts( "Created " );
  if( started ){
    f->puts( "on " );
    f->puts( started );
    f->puts( ", " );
  }
  f->puts( "using Addventure version " );
  f->puts( version );
  f->para();

  // Some information about the status of the game

  // Insert linking capabilities
  if( backlink ){

    // How do we link to other rooms
    if( listlinks ){
      f->puts( "If requested, you may list the " );
      f->puts( roomw );
      f->puts( "s that can be linked to.\n" );
    } else {
      f->puts( "To link to another " );
      f->puts( roomw );
      f->puts( ", you must know the " );
      f->puts( roomw );
      f->puts( " number.\n" );
    }
    
  } else {
    f->puts( "Linking to existing " );
    f->puts( roomw );
    f->puts( "s is not allowed.\n" );
  }

  // How many links can we create?
  if( maxopt ){
    f->puts( "You may link to no more than " );
    f->puts( itoa( maxopt ) );
    f->puts( " " );
    f->puts( roomw );
    f->puts( "s.\n" );
  } else {
    // Unlimited
  }

  // Will it send email in response
  if( null_strlen( response ) ){
    f->puts( "If requested, you will be notified when your " );
    f->puts( roomw );
    f->puts( " is expanded to.\n" );
  }

  // Can you subscribe to an email list?
  if( null_strlen( list ) ){
    f->puts( "There is a mailing list available for authors.\n" );
  }

  // Can you put your email in the signature
  if( mailsig ){
    f->puts( "Your email address can automatically " );
    f->puts( "be part of your signature.\n" );
  }

  // Insert contact information
  if( administrator ){
    f->puts( "For further information please contact: " );
    f->address( administrator );
    f->para();
  }

}

//---------------------------------------------------------------------------

void game_stats(){

  // Setting info
  SetAddv setting( filepath );
  char *roomw          = setting.roomword();

  // How many rooms are in the game? (completed and open)
  NextRoom nextroom( filepath );
  int lastroom = (int)nextroom - 1;

  // How many rooms have been completed (check the stats file)
  StatFile stats( filepath, 1 );
  int doneroom = stats.statlist()->count();

  f->header( 2, "Game Statistics" );

  running_info_line();
  
  f->puts( "There are currently " );
  f->puts( itoa( doneroom ) );
  f->puts( " " );
  f->puts( roomw );
  f->puts( "s that have been completed out of " );
  f->puts( itoa( lastroom ) );
  f->puts( " currently available.\n" );
}

//---------------------------------------------------------------------------

void list( int nn ){
  /*
   * nn == 0 : list by name
   * nn == 1 : list by number
   */

  // Setting info
  SetAddv setting( filepath );
  char *roomw          = setting.roomword();

  BackLink link( filepath, roomw );
  if( nn == 0 ){
    link.sort_name();
  } else {
    link.sort_number();
  }
  f->unnumlist( link.roombullets() );
}

//---------------------------------------------------------------------------

void list_name(){
  list( 0 );
}

//---------------------------------------------------------------------------

void list_num(){
  list( 1 );
}

//---------------------------------------------------------------------------

void generic_info(){

  copyright_info();
  f->line();
  game_stats();

}

/***************************************************************************/

main( int argc, char *argv[] ){

  typedef struct {
    char *pagename;
    char *pagelabel;
    void (*func)();
  } AboutPages;

  AboutPages pages[] = {
    {"copyright", "Copyright Info",          copyright_info},
    {"gamestat",  "Game Statistics",         game_stats},
    {"gameinfo",  "Game Info",               game_info},
    {"runinfo",   "Is the game running?",    running_info},
    {"listname",  "List linkable by name",   list_name},
    {"listnum",   "List linkable by number", list_num},
    {NULL,        NULL,                      generic_info}
  };

  urlpath  = getenv( "PATH_INFO" );
  filepath = getenv( "PATH_TRANSLATED" );

  if( argc > 1 ){
    // A parameter was given
    page = argv[1];
  } else {
    page = NULL;
  }

  int co = 0;
  int pagenumber = -1;

  // Send HTML output to the client
  f = new HtmlFile();;
  f->content_type();

  // Send header stuff
  f->title( "About this game." );
  f->header( 1, "About this game." );

  // Locate this page, if any
  while( (pages[co].pagename) && (pagenumber == -1) ){
    if( !null_strcmp( pages[co].pagename, page ) ){
      pagenumber = co;
    }
    co++;
  }
  if( pagenumber == -1 ) pagenumber = co;

  char *script = make_fname( getenv( "SCRIPT_NAME" ), urlpath );
  char *url;

  co = 0;
  f->puts( "[ " );
  while( pages[co].pagename ){
    if( co ){
      f->puts( " | " );
    }
    if( co != pagenumber ){
      url = make_url( NULL, NULL, script, pages[co].pagename );
      f->link( url, pages[co].pagelabel );
    } else {
      f->puts( pages[co].pagelabel );
    }
    co++;
  }
  f->puts( " ]" );
  f->para();

  (pages[pagenumber].func)();

}

/***************************************************************************/

/*
 * $Log: about.C,v $
 * Revision 1.10  1996/12/01 22:50:25  asf
 * Added information if the game is runnning or not.
 *
 * Revision 1.9  1996/09/29 16:03:37  asf
 * Removed room word dependency.
 *
 * Revision 1.8  1996/09/29 13:55:00  asf
 * Added mail related stuff.
 *
 * Revision 1.7  1996/08/17 00:37:37  asf
 * Added ability to list linkable rooms by name or number
 *
 * Revision 1.6  1996/08/16 04:36:38  asf
 * Added "pages"
 *
 * Revision 1.5  1996/03/22 22:21:17  asf
 * Added support for giving info about the maximum number of options.
 *
 * Revision 1.4  1996/02/25  17:44:36  asf
 * Added display of linking information
 *
 * Revision 1.3  1995/01/01  16:31:38  asf
 * Added support for the "room word"
 *
 * Revision 1.2  1994/10/03  15:01:27  asf
 * Version information now comes from version.h
 *
 * Revision 1.1  1994/10/01  19:51:42  asf
 * Initial revision
 *
 */
