/*
 * $Id: browser.C,v 1.2 1995/02/27 19:24:22 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1995  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <time.h>

#include "browser.H"
#include "addvutil.H"

#define BROWSER_SEP  "\t"

/***************************************************************************/

char *unkstrdup( char *old ){
  /*
   * Is strdup() ansi?
   *
   * Handle NULL in a fashon that I don't think anyone else wants to do.
   */

  char *kindaold = "-unknown-";
  char *ret;
  int   s;

  if( !old ){
    // Give a strange default answer if we have to duplicate nothing
    old = kindaold;
  }
  
  s = strlen( old ) + 1 ;
  ret = (char *)malloc( s );
  if( ret ){
    strcpy( ret, old );
  }
  
  return ret;
}

/***************************************************************************/

void numcat( char *s, int i ){
  char *tmp_str = itoa( i );   // The returned value is a static string
  strcat( s, tmp_str );
}

/***************************************************************************/

Browser::Browser( char *string ){
  /*
   * Parse it!
   */
  int co;

  char *tmp_string = unkstrdup( string );

  // Break up the string
  char *room_str     = strtok( tmp_string, BROWSER_SEP );
  char *browser_str  = strtok( NULL, BROWSER_SEP );

  // Store the somewhat normal ones
  _room = atoi( room_str );
  _browser = unkstrdup( browser_str );
}

//---------------------------------------------------------------------------

Browser::Browser( int room, char *browser ){

  // Copy the basic stuff
  _room = room;
  _browser = unkstrdup( browser );
  
}

//---------------------------------------------------------------------------

Browser::~Browser(){
  free( _browser );
}

//---------------------------------------------------------------------------
// Accessors

int Browser::room(){
  return _room;
}

//---------------------------------------------------------------------------

char *Browser::browser(){
  return _browser;
}

//---------------------------------------------------------------------------

char *Browser::string(){
  /*
   * The one-liner representation
   */

  static char tmp_str[1000];    //@@@

  int co;
  int num_children;

  strcpy( tmp_str, "" );
  numcat( tmp_str, _room );
  strcat( tmp_str, BROWSER_SEP );
  strcat( tmp_str, _browser );

  return tmp_str;
}

/***************************************************************************/
// This class does nothing except use its base classes.

BrowserNode::BrowserNode( char *string )
  :Browser( string ){
  /*
   * Parse it!
   */
}

//---------------------------------------------------------------------------

BrowserNode::BrowserNode( int room, char *browser )
  :Browser( room, browser ){
}

//---------------------------------------------------------------------------

BrowserNode::~BrowserNode(){
}

//***************************************************************************/

BrowserFile::BrowserFile( char *filepath, int load ){

  // It is never loaded initially, the call to load() can/will change this
  _loaded = 0;
  
  // Create the storage
  _browserlist = new List;

  // Get the name
  _filename = make_fname( filepath, "browsers", "addv" );

  // Possibly load the file
  if( load ){
    this->load();
  }
}

//---------------------------------------------------------------------------

BrowserFile::~BrowserFile(){
  free( _filename );
  delete _browserlist;
}

//---------------------------------------------------------------------------
// Accessors
List *BrowserFile::browserlist(){
  return _browserlist;
}

//---------------------------------------------------------------------------
// File actions
void BrowserFile::load(){

  FILE *f;

  char buffer[1000];
  char *status;

  BrowserNode *node;

  if( !_loaded ){
    _loaded = 1;

    if( f = fopen( _filename, "r" ) ){
      do {
        status = fgets( buffer, 1000, f );
        if( status ){
          node = new BrowserNode( buffer );
          _browserlist->append( *node );
        }
      } while( status );
      fclose( f );
    }
  }
  
}

//---------------------------------------------------------------------------

void BrowserFile::add( int room, char *browser ){

  FILE *f;

  // Create a node with the data
  BrowserNode *node = new BrowserNode( room, browser );
    
  // Append to the list if the rest of the file is already loaded
  if( _loaded ){
    _browserlist->append( *node );
  }

  // Regardless, put it at the end of the file
  set_permissions();
  if( f = fopen( _filename, "a" ) ){
    fprintf( f, "%s\n", node->string() );
    fclose( f );
  }
}

/***************************************************************************/

/*
 * $Log: browser.C,v $
 * Revision 1.2  1995/02/27  19:24:22  asf
 * Fixed problems with sttrdup() that involved trying to duplicate
 * a NULL pointer.  Renamed it to unkstrdup to indicate other abilities.
 *
 * Revision 1.1  1995/01/28  16:54:46  asf
 * Initial revision
 *
 */
