/*
 * $Id: newgame.C,v 1.8 1995/02/12 12:40:33 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1994, 1995  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * Make the initial files required for an Addventure game:
 *          1.html  - first text page
 *          2.html  - new room page
 *          2.lock  - indicates the room needs editing
 *   nextroom.addv  - the next room to be created (set to 3)
 *    options.addv  - general options for the game
 */

#include <unistd.h>
#include <stdio.h>
#include <time.h>

#include <htmlfile.H>

#include "opt.H"
#include "makeroom.H"
#include "addvutil.H"
#include "nextroom.H"
#include "setaddv.H"
#include "lockfile.H"

/***************************************************************************/

void usage( char *program ){
  // @@@
  fprintf( stderr, "No bozo\n" );
}

/***************************************************************************/

int make_first( char *filepath, char *urlpath ){
  int ret;      // Return value
  
  char *file;   // The full actual path including file name
  FILE *room;

  SetAddv setting( filepath );    // The game defaults and values
  char *gametitle = setting.game_title();   // Is there a name for the game?
  char *roomword  = setting.roomword();

  char *roomtitle = "Welcome to Addventure";
  char *roomnum   = "1";

  char *title;     // The title of the page
  int  titlelen;
  char *roomline;  // The line that has the room number
  int  roomlinelen;

  if( gametitle ){
    titlelen = strlen(gametitle) + strlen(roomtitle) + strlen(roomword) + 20;
    title = (char *)malloc( titlelen );
    sprintf( title, "%s: %s [%s 1]", gametitle, roomtitle, roomword );
    
    roomlinelen = strlen(gametitle) + strlen(roomword) + strlen(roomnum) + 10;
    roomline = (char *)malloc( roomlinelen );
    sprintf( roomline, "%s - %s %s", gametitle, roomword, roomnum );
  } else {
    // Just normal copies
    titlelen = strlen(roomtitle) + strlen(roomword) + 20;
    title = (char *)malloc( titlelen );
    sprintf( title, "%s [%s 1]", roomtitle, roomword );

    roomlinelen = strlen(roomword) + strlen(roomnum) + 10;
    roomline = (char *)malloc( roomlinelen );
    sprintf( roomline, "%s %s", roomword, roomnum );
  }

  // @@@ Should do error checking
  file = make_fname( filepath, 1, "html" );
  set_permissions();
  if( room = fopen( file, "w" ) ){
    // File opened
    HtmlFile r( room );
    r.title( title );
    r.header( 1, "Welcome to Addventure" );
    r.header( 2, roomline );

    r.puts( "Game started on ");
    time_t t = time(NULL);
    r.puts( ctime( &t ) );  r.para();

    r.puts( "Prepare to PLAY!\n" );

    char *pathroom = make_fname( NULL, "2.html" );
    char *url      = make_url( NULL, NULL, pathroom );
    r.link( url, "Begin" );
    free( pathroom );
    free( url );

    fclose( room );

    ret = 1;
  } else {
    // Create failed
    ret = 0;
  }
  
  free( file );

  return ret;
}

/***************************************************************************/

main( int argc, char *argv[] ){

  Opt cla( argc, argv );   // Command line arguments

  char *filepath;   // The path of the game files
  char *urlpath;    // What the server things the path is
  char *binpath;    // Where the binary is

  filepath = cla.get_value( "-filepath" );
  urlpath = cla.get_value( "-urlpath" );
  binpath = cla.get_value( "-binpath" );

  if( !filepath || !urlpath || !binpath ){
    // No path found
    usage( argv[0] );
    exit(1);
  } else {
    // Path found

    // @@@ No error checking!?!
    
    /*
     * Create the data file that hold various information that is
     * mostly read.
     */
    SetAddv  setting( filepath );
    setting.set( cla );

    /*
     * Make the rooms and locks
     */
    make_first( filepath, urlpath );
    make_room( filepath, urlpath, binpath, setting.roomword(), 2, 1 );
    LockFile lockfile( filepath, 2 );
    lockfile.create( 1 );

    /*
     * Create the data file that holds the number of the next room
     * to create.
     */
    NextRoom nextroom( filepath );
    nextroom = 3;

    exit(0);
  }
}

/***************************************************************************/

/*
 * $Log: newgame.C,v $
 * Revision 1.8  1995/02/12  12:40:33  asf
 * Changes to "correctly" support relative addressing.
 *
 * Revision 1.7  1995/01/22  16:27:45  asf
 * Changed to support a relative path to room 2.
 *
 * Revision 1.6  1995/01/01  16:43:41  asf
 * Added support for the "room word"
 *
 * Revision 1.5  1994/10/01  15:44:20  asf
 * Switched to use of LockFile and OpenFile.
 *
 * Revision 1.4  1994/09/16  21:34:11  asf
 * Fixed a problem with the game title
 *
 * Revision 1.3  1994/09/16  16:06:21  asf
 * The game title is now part of the title and header.
 *
 * Revision 1.2  1994/09/14  13:46:01  asf
 * Use the SetAddv object instead of the Setting object
 *
 * Revision 1.1  1994/08/04  20:34:55  asf
 * Initial revision
 *
 */
