/*
 * $Id: option.C,v 1.4 1995/05/26 16:10:07 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1994  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include <string.h>

#include "option.H"

/***************************************************************************/

Option::Option( char *desc, int room ){
  _desc = NULL;       // Ensure we know theres nothing there
  set( desc, room );
}

//---------------------------------------------------------------------------

Option::~Option(){
  if( _desc ){
    free( _desc );
  }
}

//---------------------------------------------------------------------------
// Accessors

char *Option::description(){
  return _desc;
}

//---------------------------------------------------------------------------

int Option::room(){
  return _room;
}

//---------------------------------------------------------------------------
// Set values

void Option::set( char *desc, int room ){
  set( desc );
  set( room );
}

//---------------------------------------------------------------------------

void Option::set( char *desc ){

  // Remove anything that might have been there
  if( _desc ){
    free( _desc );
  }

  // Either allocate space for the new string, or just store a NULL there
  if( desc ){
    int l = strlen( desc ) + 1;
    _desc = (char *)malloc( l );
    if( _desc ){
      strcpy( _desc, desc );
    } else {
      _desc = NULL;
    }
  } else {
    _desc = NULL;
  }
}

//---------------------------------------------------------------------------

void Option::set( int room ){
  _room = room;
}

/***************************************************************************/

OptionArray::OptionArray( int size ){
  /*
   * Allocate "size" spaces from 1 to "size"
   */

  if( size > 0 ){
    _size = size;
    _o = new Option[size];
  } else {
    _o = NULL;
  }
}

//---------------------------------------------------------------------------

OptionArray::~OptionArray(){
  /*
   * Free all options
   */
  if( _o ){
    delete [] _o;
  }
}

//---------------------------------------------------------------------------

Option *OptionArray::operator[]( int index ){
  Option *ret;
  
  if( (_o) && (index>0) && (index<=_size) ){
    ret = &(_o[index-1]);
  } else {
    ret = NULL;
  }

  return ret;
}

//---------------------------------------------------------------------------

int OptionArray::size(){
  return _size;
}

/***************************************************************************/

/*
 * $Log: option.C,v $
 * Revision 1.4  1995/05/26  16:10:07  asf
 * Changes in the constructor to fix compile problems.
 *
 * Revision 1.3  1994/10/03  12:40:10  asf
 * ~OptionArray() now does a deletion of the entire array of options, instead
 * of just one option.  (C-- has stupid syntax.)
 *
 * Revision 1.2  1994/08/26  16:51:30  asf
 * Added the size() accessor
 *
 * Revision 1.1  1994/08/26  16:44:39  asf
 * Initial revision
 *
 */
