/*
 * $Id: setaddv.H,v 1.15 1996/09/29 13:23:44 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1991-94, 1995, 1996  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _SETADDV_H
#define _SETADDV_H

#include "setting.H"

/***************************************************************************/

class SetAddv:public Setting{
public:  
  // Constructor/destructor
  SetAddv( char *path );
  ~SetAddv();

  // Parse everything else from the command line
  void set( Opt &command_line );

  // Query with some processing
  char *getPathLabel( char *label );

  // Get some values
  char *version();      // What version was this game created with?
  char *date_started(); // When was the game started?
  int running();        // Is the game open?
  int backlink();       // Should we backlink at all?
  int linkdefault();    // Should we allow linking to a room by default?
  int listlinks();      // Should we allow listing of links?
  int evil_html();      // Should we allow various HTML?
  int cleanup_after();  // When should we restore a stale room?
  int maxoptions();     // How many choices can they make?  (0 == infinite)
  int goback();         // When should we provide a link to the parent?
  char *game_title();   // Game title  (NULL if none)
  char *contact();      // Who is in charge of this game?
  char *roomword();     // What word do we use for "room"?
  char *mailhelper();   // A helper script that will send mail
  char *listhelper();   // A helper script that will subscribe to a maillist
  int mailsig();        // Should we allow mailto: URLs around the signature?

  // These are for Java applets
  // Without a parameter, it returns the line indicating what other
  //   lines to look for.  With a parameter it will return that
  //   value.

  char *appletelement( char *element = NULL );
  char *paramname( char *name = NULL );

  // Should a component print debugging information?  Reads a space
  //   separated list of strings, and returns true if any of them
  //   are a leading substring of the component passed as a parameter.

  int debug( char *component );

  
private:
  char *_path;
  int  getboolean( char *label, int def = 0 );
  void setboolean( Opt &command_line, char *cl_value,
                   char *label, int def );
};

/***************************************************************************/

#endif

/*
 * $Log: setaddv.H,v $
 * Revision 1.15  1996/09/29 13:23:44  asf
 * Added listhelper() and mailsig() methods.
 *
 * Revision 1.14  1996/07/10 21:33:41  asf
 * Added debug()
 *
 * Revision 1.13  1996/07/10 13:04:47  asf
 * Added appletelement() and paramname()
 *
 * Revision 1.12  1996/03/25  20:14:36  asf
 * Added _path
 * Added getPathLabel() method
 *
 * Revision 1.11  1996/03/24  17:44:20  asf
 * Added running() to determine if the game is open or not.
 *
 * Revision 1.10  1996/03/23  16:47:49  asf
 * Added mailhelper() method
 *
 * Revision 1.9  1996/03/22  22:20:36  asf
 * Added maxoptions() method to return the number of choices a person
 * can add.
 *
 * Revision 1.8  1996/02/25  17:37:58  asf
 * Added listlinks()
 *
 * Revision 1.7  1995/01/28  16:59:50  asf
 * Changed the "anchor option" to "evil_html".
 *
 * Revision 1.6  1995/01/01  16:43:11  asf
 * Added support for the "room word"
 *
 * Revision 1.5  1994/12/17  20:11:27  asf
 * Added the ability to put or deny anchors in optiosn.
 *
 * Revision 1.4  1994/10/01  19:35:51  asf
 * Added version(), date_started(), and contact() accessors.
 *
 * Revision 1.3  1994/09/16  14:17:08  asf
 * Added various accessor functions.
 *
 * Revision 1.2  1994/09/15  11:24:19  asf
 * Changed parameter name from "default" to "def"
 *
 * Revision 1.1  1994/09/14  13:49:36  asf
 * Initial revision
 *
 */
