/*
 * $Id: stat.C,v 1.11 1995/10/10 07:48:39 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1994, 1995  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <time.h>

#include "stat.H"
#include "option.H"
#include "addvutil.H"

#define STAT_SEP  "\t"
#define CHILD_SEP ","
// @@@ right now i assume that CHILD_SEP is a character

/***************************************************************************/

char *sstrdup( char *old ){
  /*
   * Is strdup() ansi?
   */

  char *ret;
  int   s;

  s = strlen( old ) + 1 ;
  ret = (char *)malloc( s );
  if( ret ){
    strcpy( ret, old );
  }
  
  return ret;
}

/***************************************************************************/

void numcat( char *s, int i ){
  char *tmp_str = itoa( i );   // The returned value is a static string
  strcat( s, tmp_str );
}

/***************************************************************************/

int charcount( char *s, char *c ){
  /*
   * Count the number of times character "c" occurs in string "s"
   * if string "s" exists.
   *
   * @@@ Right now I'm assuming that "c" is a character
   */
  
  int ret = 0;
  int sl;
  int co;

  if( s ){
    sl = strlen( s );
    for( co=0; co<sl; co++ ){
      if( s[co] == c[0] ) ret++;
    }
  }

  return ret;
}

/***************************************************************************/

Stat::Stat( char *string ){
  /*
   * Parse it!
   */
  int co;

  char *tmp_string = sstrdup( string );

  // Break up the string
  char *room_str = strtok( tmp_string, STAT_SEP );
  char *children_str = strtok( NULL, STAT_SEP );
  char *dtg_str = strtok( NULL, STAT_SEP );
  char *host_str = strtok( NULL, STAT_SEP );
  char *author_str = strtok( NULL, STAT_SEP );

  if( room_str && children_str && dtg_str && host_str ){

    // Store the somewhat normal ones
    _valid = 1;
    _room = atoi( room_str );
    _dtg = sstrdup( dtg_str );
    _host = sstrdup( host_str );
    _author = sstrdup( author_str );

    // Allocate an array that is big enough and store it there
    int children_count = 0;
    int total_children = charcount( children_str, CHILD_SEP ) + 1;
    OptionArray tmp_children( total_children );
    
    // Store the list of children
    char *tmp_child_str;
    int   tmp_child;
    tmp_child_str = strtok( children_str, CHILD_SEP );
    while( tmp_child_str ){
      // Get the value and where to store it
      tmp_child = atoi( tmp_child_str );
      children_count++;

      // Store it
      tmp_children[children_count]->set( tmp_child );

      // Get the next one
      tmp_child_str = strtok( NULL, CHILD_SEP );
    }

    // Make the real list
    _children = new OptionArray( children_count );

    // Fill it
    for( co=1; co<=children_count; co++ ){
      ((*_children)[co])->set( tmp_children[co]->room() );
    }

  } else {
    _valid = 0;

    _room = -1;
    _dtg = sstrdup( "" );
    _host = sstrdup( "" );
    _author = sstrdup( "" );
    _children = NULL;
    
  }
}

//---------------------------------------------------------------------------

Stat::Stat( int room, OptionArray &children,
            char *host, char *author,
            char *dtg ){

  int co;

  _valid = 1;

  // Time stuff
  time_t ti;
  struct tm *t;
  char tmp_dtg[17];

  // Copy the basic stuff
  _room = room;
  _host = sstrdup( host );
  _author = sstrdup( author );
  if( dtg ){
    _dtg = sstrdup( dtg );
  } else {
    // Create the dtg
    ti = time( &ti );
    t = localtime( &ti );
    strftime( tmp_dtg, 16, "%Y%m%d-%H%M%S", t );
    _dtg = sstrdup( tmp_dtg );
  }

  // Create the new options
  int s = children.size();
  _children = new OptionArray( s );

  // Copy the options
  for( co=1; co<=s; co++ ){
    // Set each item in new array with just the room number from the old one
    ((*_children)[co])->set( (children[co])->room() );
  }
}

//---------------------------------------------------------------------------

Stat::~Stat(){
  if( _children ){
    delete _children;
  }
  free( _host );
  free( _author );
  if( _dtg ){
    free( _dtg );
  }
}

//---------------------------------------------------------------------------
// Accessors

int Stat::valid(){
  return _valid;
}

//---------------------------------------------------------------------------

int Stat::room(){
  return _room;
}

//---------------------------------------------------------------------------

OptionArray *Stat::children(){
  return _children;
}

//---------------------------------------------------------------------------

char *Stat::host(){
  return _host;
}

//---------------------------------------------------------------------------

char *Stat::author(){
  return _author;
}

//---------------------------------------------------------------------------

char *Stat::date(){
  return _dtg;
}

//---------------------------------------------------------------------------

char *Stat::string(){
  /*
   * The one-liner representation
   */

  static char tmp_str[1000];    //@@@

  int co;
  int num_children;

  strcpy( tmp_str, "" );
  numcat( tmp_str, _room );
  strcat( tmp_str, STAT_SEP );

  // Add the children
  num_children = _children->size();
  for( co=1; co<num_children; co++ ){
    numcat( tmp_str, ((*_children)[co])->room() );
    strcat( tmp_str, CHILD_SEP );
  }
  numcat( tmp_str, ((*_children)[num_children])->room() );
  
  // Add the rest of the crap
  strcat( tmp_str, STAT_SEP );
  strcat( tmp_str, _dtg );
  strcat( tmp_str, STAT_SEP );
  strcat( tmp_str, _host );
  strcat( tmp_str, STAT_SEP );
  strcat( tmp_str, _author );

  return tmp_str;
}

/***************************************************************************/
// This class does nothing except use its base classes.

StatNode::StatNode( char *string )
  :Stat( string ){
  /*
   * Parse it!
   */
}

//---------------------------------------------------------------------------

StatNode::StatNode( int room, OptionArray &children,
                    char *host, char *author,
                    char *dtg )
  :Stat( room, children, host, author, dtg ){
}

//---------------------------------------------------------------------------

StatNode::~StatNode(){
}

//***************************************************************************/

StatFile::StatFile( char *filepath, int load ){

  // It is never loaded initially, the call to load() can/will change this
  _loaded = 0;
  
  // Create the storage
  _statlist = new List;

  // Get the name
  _filename = make_fname( filepath, "stat", "addv" );

  // Possibly load the file
  if( load ){
    this->load();
  }
}

//---------------------------------------------------------------------------

StatFile::~StatFile(){
  free( _filename );
  delete _statlist;
}

//---------------------------------------------------------------------------
// Accessors
List *StatFile::statlist(){
  return _statlist;
}

//---------------------------------------------------------------------------
// File actions
void StatFile::load(){

  FILE *f;
  int line = 0;

  char buffer[1000];
  char *status;

  StatNode *node;

  if( !_loaded ){
    _loaded = 1;

    if( f = fopen( _filename, "r" ) ){
      do {
        status = fgets( buffer, 1000, f );
        line++;
        if( status ){
          node = new StatNode( buffer );
          if( node->valid() ){
            _statlist->append( *node );
          } else {
            // fprintf( stderr, "StatFile: error in line %d\n", line );
          }
        }
      } while( status );
      fclose( f );
    }
  }
  
}

//---------------------------------------------------------------------------

void StatFile::add( int room, OptionArray &children,
                    char *host, char *author ){

  FILE *f;

  // Create a node with the data
  StatNode *node = new StatNode( room, children, host, author );
    
  // Append to the list if the rest of the file is already loaded
  if( _loaded ){
    _statlist->append( *node );
  }

  // Regardless, put it at the end of the file
  set_permissions();
  if( f = fopen( _filename, "a" ) ){
    fprintf( f, "%s\n", node->string() );
    fclose( f );
  }
}

/***************************************************************************/

/*
 * $Log: stat.C,v $
 * Revision 1.11  1995/10/10  07:48:39  asf
 * Added checks to ensure a line read from a file is valid and
 * only insert it if it is.
 *
 * Revision 1.10  1995/05/26  16:11:44  asf
 * Changes in constructors to deal with compile problems.
 *
 * Revision 1.9  1995/05/07  14:57:54  asf
 * Fix to allow for an unlimited number of children.  (Previous restriction was
 * 100 children.)
 *
 * Revision 1.8  1994/10/03  12:41:22  asf
 * sstrdup() used an incorrect size to allocate space.
 *
 * Revision 1.7  1994/09/16  17:36:27  asf
 * StatFile::_loaded was not being initialized to anything, so occasionally
 * load() would think it was already loaded and assume the list was already
 * allocated.  This can cause a crash later.  Fixed by initializing _loaded
 * to 0 always (it gets set by load() ).
 *
 * Revision 1.6  1994/09/01  19:36:32  asf
 * Increased the tmp_dtg in the Stat constructor.  I don't know why
 * this works.
 *
 * Revision 1.5  1994/09/01  18:30:03  asf
 * We need to actually _return_ the value from Stat::string()
 *
 * Revision 1.4  1994/09/01  18:03:02  asf
 * Fixed a problem with numcat() where it would free a static space.
 *
 * Revision 1.3  1994/08/30  22:04:26  asf
 * Fixed a looping problem that left a 0 at the end of the children list
 *
 * Revision 1.2  1994/08/30  21:55:51  asf
 * Added figuring out the date string
 *
 * Revision 1.1  1994/08/30  21:12:56  asf
 * Initial revision
 *
 */
