/*
 * $Id: tokenizer.C,v 1.3 1996/08/15 23:47:54 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1996  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include <string.h>

#include "tokenizer.H"
#include "addvutil.H"

/***************************************************************************/

Tokenizer::Tokenizer( char *string ){
  v_ptr = v_string = null_strdup( string );
  v_tokens = null_strdup( " \t\r\n" );
  v_finished = 0;
}

//---------------------------------------------------------------------------

Tokenizer::Tokenizer( char *string, char *tokens ){
  v_ptr = v_string = null_strdup( string );
  v_tokens = null_strdup( tokens );
  v_finished = 0;
}

//---------------------------------------------------------------------------

Tokenizer::Tokenizer( char *string, char token ){
  v_ptr = v_string = null_strdup( string );
  v_tokens = chartostr( token );
  v_finished = 0;
}

//---------------------------------------------------------------------------

Tokenizer::~Tokenizer(){
  free( v_string );
  free( v_tokens );
}

/***************************************************************************/

char *Tokenizer::getnext(){
  return getnext( v_tokens );
}

//---------------------------------------------------------------------------

char *Tokenizer::getnext( char *tokens ){
  char *ret = v_ptr;

  // Check to see if we're done
  if( *v_ptr == '\0' ){
    v_finished = 1;
  }

  if( !v_finished ){

    // Advance to the end or a separator
    while( *v_ptr && !strchr( tokens, *v_ptr ) ){
      v_ptr++;
    }

    if( *v_ptr == '\0' ){
      // If we're at the end, lets say so
      v_finished = 1;
    } else {
      // Otherwise, null terminate, and we'll do more later
      *v_ptr = '\0';
      v_ptr++;
    }
  } else {
    ret = NULL;
  }

  return ret;
}

//---------------------------------------------------------------------------

char *Tokenizer::getnext( char token ){
  return getnext( chartostr( token ) );
}

/***************************************************************************/

int Tokenizer::finished(){
  return v_finished;
}

/***************************************************************************/

/*
 * $Log: tokenizer.C,v $
 * Revision 1.3  1996/08/15 23:47:54  asf
 * Load stdlib.h
 *
 * Revision 1.2  1996/08/06 02:01:45  asf
 * Detect when we have reached the end of parsing before we even try to
 * read the next token
 *
 * Revision 1.1  1996/08/04 01:19:27  asf
 * Initial revision
 *
 */
