/*
 * $Id: undo.C,v 1.3 1996/08/15 23:46:18 asf Exp $
 */

/*
    This file is part of Addventure
    Copyright (C) 1994  Allen S. Firstenberg
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "undo.H"
#include "tokenizer.H"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#define SEPCHAR '-'
#define NOJAVA "j"

/***************************************************************************/

// Constructor/Destructor
Undo::Undo( char *coded ){
  char *roomstr;
  char *seperator;
  char *parentstr;
  char *paramstr;

  if( coded ){

    // Create the tokenizer
    Tokenizer tokens( coded, SEPCHAR );
    
    // Parse
    // Get the room
    roomstr = tokens.getnext();

    // Get the parent
    if( !tokens.finished() ){
      // There is more.  Get them
      parentstr = tokens.getnext();
    } else {
      parentstr = "0";
    }

    // Get java or not.  A "j" indicates to block java
    if( !tokens.finished() ){
      // Lets see what we have
      paramstr = tokens.getnext();
      if( !strcmp( paramstr, NOJAVA ) ){
        // Block java
        v_nojava = 1;
      }
    } else {
      // Nothing.  So don't block
      v_nojava = 0;
    }
    
    // Get the values
    v_room = atoi( roomstr );
    v_parent = atoi( parentstr );
    
  } else {
    // Set everything to zero
    v_room = 0;
    v_parent = 0;
  }
}

//---------------------------------------------------------------------------

Undo::Undo( int room, int parent, int nojava ){
  v_room = room;
  v_parent = parent;
  v_nojava = nojava;
}

//---------------------------------------------------------------------------

Undo::~Undo(){
}

//---------------------------------------------------------------------------
// Accessors

char *Undo::coded_value(){
  /*
   * Return a newly allocated string
   */
  char *ret;

  ret = (char *)malloc( 50 );    // @@@

  if( !v_nojava ){
    // Yes, include java work.  This is the default.
    sprintf( ret, "%d%c%d", v_room, SEPCHAR, v_parent );
  } else {
    // No java stuff
    sprintf( ret, "%d%c%d%c%s", v_room, SEPCHAR, v_parent,
                                SEPCHAR, NOJAVA );
  }

  return ret;
}

//---------------------------------------------------------------------------

int Undo::room(){
  return v_room;
}

//---------------------------------------------------------------------------

int Undo::parent(){
  return v_parent;
}

//---------------------------------------------------------------------------

int Undo::dojava(){
  return !v_nojava;
}

/***************************************************************************/

/*
 * $Log: undo.C,v $
 * Revision 1.3  1996/08/15 23:46:18  asf
 * Changed the tokenization method.
 * The cookie returned can now give a hint if java is desired or not, and
 * can process this cookie.
 *
 * Revision 1.2  1996/08/06 02:02:24  asf
 * Use the tokenizer instead of hand parsing
 *
 * Revision 1.1  1994/08/20 22:06:30  asf
 * Initial revision
 *
 */
